<?php

namespace Drupal\smart_date_recur\Plugin\QueueWorker;

use Drupal\smart_date_recur\Entity\SmartDateRule;
use Drupal\Core\Queue\QueueWorkerBase;

/**
 * Updates a rule's instances.
 *
 * @QueueWorker(
 *   id = "smart_date_recur_rules",
 *   title = @Translation("Smart Date Recur rules refresh"),
 *   cron = {"time" = 60}
 * )
 */
class RecurRuleUpdate extends QueueWorkerBase {

  /**
   * {@inheritdoc}
   */
  public function processItem($item) {
    if (empty($item->data)) {
      return;
    }
    $entity_manager = \Drupal::entityManager($item->entity_type);
    $entity_storage = $entity_manager
      ->getStorage($item->entity_type);

    $entity = $entity_storage
      ->load($item->entity_id);

    foreach($item->data as $field_name => $rules) {
      $field_values = $entity->get($field_name)->getValue();
      $processed = [];
      $rules_processed = [];
      foreach ($field_values as $delta => $row) {
        if (empty($row['rrule']) || !isset($rules[$row['rrule']])) {
          // Add directly to our array.
          $processed[] = $row;
          continue;
        }
        if (isset($rules_processed[$row['rrule']])) {
          // Already handled this rule, so skip this row.
          continue;
        }
        $rule = SmartDateRule::load($row['rrule']);
        $month_limit = $rule->getFieldSettings('month_limit');
        $before = strtotime('+' . (int) $month_limit . ' months');
        $new_instances = $rule->getRuleInstances($before);
        if (count($new_instances) == count($rules[$row['rrule']])) {
          // No additional instances so use what we have.
          $processed[] = $row;
          unset($rules[$row['rrule']]);
          continue;
        }
        // Add the generated the instances.
        foreach ($new_instances as $new_instance) {
          $row['value'] = $new_instance['value'];
          $row['end_value'] = $new_instance['end_value'];
          $processed[] = $row;
        }
        $rule->set('instances', ['data' => $new_instances]);
        $rule->save();
        // Track that this rule has been processed.
        $rules_processed[$row['rrule']] = $row['rrule'];
      }
      // Update the entity with our new values.
      $entity->set($field_name, $processed);
    }
    if (!empty($rules_processed)) {
      $entity->save();
    }
  }

}
